<?php
/**
* Handles the "setup".
*
* This class provides all methods neccessary to "setup" Phpdoc and check the 
* current setup.
* 
* @version 	0.1alpha
* @author		Ulf Wendel <ulf@redsys.de>
*/
class PhpdocSetupHandler extends PhpdocArgvHandler {
	
	/**
	* Name of the files that get parsed.
	* @var		array		$files
	* @access	private
	*/												
	var $files = array();	
	
	/**
	* Name of a directory that gets parsed.
	* @var		string	$directory
	* @access	private
	*/
	var $directory = "";
	
	/**
	* Name of the target directory.
	* @var		string	$target
	* @access	private
	*/								
	var $target;	

	/**
	* Sets the directory that gets parsed
	* @param	string	$directory	name of a directory
	* @return	bool		$ok
	* @throws	runtime_wrongparam
	* @access	public
	*/
	function setDirectory($directory) {
		if (""==$directory) {
			$this->err = new exception_runtime_wrongparam("No directory specified.", 5, __FILE__, __LINE__);
			return false;
		}
		$this->directory = $directory;
		return true;
	} // end func setDirectory
	
	/**
	* Sets the files that gets parsed
	* @param	string	$files	name of a file
	* @param	array		$files	array of filenames ($k => $filename)
	* @return bool		$ok
	* @throws	runtime_wrongparam
	* @access	public
	*/
	function setFiles($files) {
		if (""==$files) {
			$this->err = new exception_runtime_wrongparam("No files specified.", 2, __FILE__, __LINE__);
			return false;
		}	
		
		if (!is_array($files))
			$files = array($files);
		
		reset($files);
		while (list($k, $filename)=each($files))
			$this->files[] = trim($filename);
			
		return true;
	} // end func setFiles 
	
	/**
	* Sets the target where the generated files are saved. Tries to create 
	* the directory if not existant.
	* @param	string	$target
	* @return	bool		$ok 
	* @throws runtime_wrongparam
	* @access	public
	*/
	function setTarget($target) {
		if (""==$target) {
			$this->err = new exception_runtime_wrongparam("No target specified.", 3, __FILE__, __LINE__);
			return false;
		}
		
		if (!is_dir($target)) {
			$ok = @mkdir($target, 0755);
			if (!$ok) {
				$this->err = new exception_io("setTarget(), can't create a directory '$target'.", 4, __FILE__, __LINE__);
				return false;
			}
		}
			
		$this->target = $target;
		return true;
	} // end func setTarget

	/**
	* Sets the path to the source files
	* @param	string	$path
	* @access	public
	*/	
	function setPath($path) {
		$this->path = $path;
	} // end func setPath
	
	/**
	* Checks the current status of the object. Are all necessary informations 
	* to start parsing available?
	* @param	array		$errors
	* @access	private
	* @return	array		$errors
	*/
	function checkStatus($errors="") {
		if (!is_array($errors))
			$errors = array();
				
		if (0==count($this->files) && ""==$this->directory) 
			$errors[] = array (
													"msg" 	=> "No source files or source directory specified.",
													"type"	=> "misconfiguration",
													"errno"	=> 6
												);
												
		if (0!=count($this->files) && ""!=$this->directory) 
			$errors[] = array(
													"msg"		=> "Define eighter some files or a diretory.",
													"type"	=> "misconfiguration",
													"errno"	=> 7
											);
		
		return $errors;
	} // end func checkStatus

} // end class PhpdocSetupHandler
?>