<?php
/**
* Exception of the type "notice". Use the types notice, warning and error only 
* if io and runtime do not fit. "notice" is used to indicate minor problems
* and hints, e.g. hints on using your class more effectively.
* @author		Ulf Wendel <ulf@redsys.de>
* @version	0.1
*/
class exception_notice extends exception_handler {
	/**
	* Create an exception of the type "notice". 
	* @param	string $message=""	text describing the exception
	* @param	int		 $errno=0			unique identifier
	* @param	string $file=""			name of the file that throws the exception, use __FILE__
	* @param	int		 $line=0 			current line number, use __LINE__
	* @param	bool	 $auto_handle if true handles the exception automatically 
	*/
	function exception_notice($message="", $errno=0, $file="", $line=0, $auto_handle=false) {
		$this->exception($message, $errno, "notice", $file, $line);
		if (true == $auto_handle)
			$this->handle();
	} // end constructor
	
} // end class exception_notice

/**
* Exception of the type "warning". Use the types notice, warning and error only 
* if io and runtime do not fit. "warning" is used to indicate serious problems
* that don't need to halt excecution but should be shown. Only exceptions 
* of the type "error" should halt execution.
* @author		Ulf Wendel <ulf@redsys.de>
* @version	0.1
*/
class exception_warning extends exception_handler {
	/**
	* Create an exception of the type "warning". 
	* @param	string $message=""	text describing the exception
	* @param	int		 $errno=0			unique identifier
	* @param	string $file=""			name of the file that throws the exception, use __FILE__
	* @param	int		 $line=0 			current line number, use __LINE__
	* @param	bool	 $auto_handle if true handles the exception automatically 
	*/
	function exception_warning($message="", $errno=0, $file="", $line=0, $auto_handle=false) {
		$this->exception($message, $errno, "warning", $file, $line);
		if (true == $auto_handle)
			$this->handle();
	} // end constructor
} // end class exception_warning

/**
* Exception of the type "error". Use the types notice, warning and error only 
* if io and runtime do not fit. "error" is used to indicate serious problems that 
* should stop excecution. 
* @author		Ulf Wendel <ulf@redsys.de>
* @version	0.1
*/
class exception_error extends exception_handler {
	/**
	* Create an exception of the type "error". 
	* @param	string $message=""	text describing the exception
	* @param	int		 $errno=0			unique identifier
	* @param	string $file=""			name of the file that throws the exception, use __FILE__
	* @param	int		 $line=0 			current line number, use __LINE__
	* @param	bool	 $auto_handle if true handles the exception automatically 
	*/
	function exception_error($message="", $errno=0, $file="", $line=0, $auto_handle=false) {
		$this->exception($message, $errno, "error", $file, $line);
		if (true == $auto_handle)
			$this->handle();
	} // end constructor
} // end class exception_error

/**
* Exception of the type "io". Use this type whenever an error occurs while dealing 
* with IO operations, e.g. fopen(), xy_connect() etc. Don't use notice, warning or
* error for this purpose.
* @author		Ulf Wendel <ulf@redsys.de>
* @version	0.1
*/
class exception_io extends exception_handler {
	/**
	* Create an exception of the type "io". 
	* @param	string $message=""	text describing the exception
	* @param	int		 $errno=0			unique identifier
	* @param	string $file=""			name of the file that throws the exception, use __FILE__
	* @param	int		 $line=0 			current line number, use __LINE__
	* @param	bool	 $auto_handle if true handles the exception automatically 
	*/
	function exception_io ($message="", $errno=0, $file="", $line=0, $auto_handle=false) {
		$this->exception($message, $errno, "io", $file, $line);
		if (true == $auto_handle)
			$this->handle();
	} // end constructor
} // end class exception_io

/**
* Exception of the type "runtime". Use this type whenever the API of your class 
* gets used in a wrong way, e.g. WRONG_PARAM_COUNT.
* @author		Ulf Wendel <ulf@redsys.de>
* @version  0.1
*/
class exception_runtime extends exception_handler {
	/**
	* Create an exception of the type "runtime". 
	* @param	string $message=""	text describing the exception
	* @param	int		 $errno=0			unique identifier
	* @param	string $file=""			name of the file that throws the exception, use __FILE__
	* @param	int		 $line=0 			current line number, use __LINE__
	* @param	bool	 $auto_handle if true handles the exception automatically 
	*/
	function exception_runtime($message="", $errno=0, $file="", $line=0, $auto_handle=false) {
		$this->exception($message, $errno, "runtime", $file, $line);
		if (true == $auto_handle)
			$this->handle();
	} // end constructor
} // end class exception_runtime 

/**
* Runtime exception used to indicate wrong parameter calls or illegal parameters
* @author		Ulf Wendel <ulf@redsys.de>
* @version  0.1
*/
class exception_runtime_wrongparam extends exception_runtime {
	/**
	* Create an exception of the type "runtime_wrongparam". 
	* @param	string $message=""	text describing the exception
	* @param	int		 $errno=0			unique identifier
	* @param	string $file=""			name of the file that throws the exception, use __FILE__
	* @param	int		 $line=0 			current line number, use __LINE__
	* @param	bool	 $auto_handle if true handles the exception automatically 
	*/
	function exception_runtime_wrongparam($message="", $errno=0, $file="", $line=0, $auto_handle=false) {
		$this->exception($message, $errno, "runtime_wrongparam", $file, $line);
		if (true == $auto_handle)
			$this->handle();
	} // end constructor
} // end class exception_runtime_wrongparam
?>