<?php
/**
* Parses phpcode to extract classes and their documentation.
*
* @version	0.1alpha
* @author		Ulf Wendel, ulf@redsys.de
*/
class PhpdocClassParser extends PhpdocFunctionParser {

  /**
  * Array shared by several functions holding the informations found in the code.
  * @var    array $classes
  * @access private
  */ 
  var $classes = array();
	
	/**
	* Just calls Setup()
	* @see	Setup()
	*/
	function PhpdocClassParser() {
		$this->Setup();			
	} // end constructor
	
	/**
	* Analyse the given PHPCode and Phpdoc paragraphs.
	* @param	string	$phpcode	code to analyse
	* @return	array		$classes
	* @throws runtime_wrongparam
	* @access	public
	*/
	function analyseClasses($paragraphs) {
		$this->classes = array();
		
		reset($paragraphs);
		while (list($k, $paragraph)=each($paragraphs)) 
			$this->analyseClassParagraph($paragraph);
			
		return $this->classes;	
	} // end func analyseClasses
	
	/**
	* Analyses a singe 
	*
	*/
	function analyseClassParagraph($paragraph) {
		
		$class = array();
		
		list($classname, $extends)	= $this->getClassnamesFromRegs($paragraph["regs"]);
		$class["classname"]		= $classname;
		$class["extends"] 		= $extends;

		list($shortdesc, $fulldesc) = $this->getDescription($paragraph["phpdoc"]);
		$class["shortdesc"]		= $shortdesc;
		$class["desc"]				= $fulldesc;
		
		$tags = $this->getTags($paragraph["phpdoc"]);			
		reset($tags);
		while (list($k, $tag)=each($tags)) {
			$tagname = $tag["tag"];
			switch ($tagname) {
				case "@author": 
				case "@version":
				case "@since":
				case "@abstract":
					$class[substr($tagname, 1)] = $tag["value"];
					break;
					
				default:
					$class["ignored"][] = $tag;
					break;
			}
		}
		
		$this->classes[$classname] = $class;
	} // end func analyseClassParagraph
	
	/**
	* Finds classes and their PHP code in the given string.
	* 
	* Phpdoc currently works on per class base. That means analysing PHP source 
	* always starts with extracting classes and their PHP code from the 
	* given PHP source.
	* 
	* @param	string	$phpcode
	* @return	array		$classes	Array of PHP code, indexed by the classname
	* @see		getClassnamesFromRegs()
	*/
	function getClasscode($phpcode) {
		
		$php_class 				= "/".substr($this->PHP_CLASS, 2);
		$php_class_extend = "/".substr($this->PHP_CLASS_EXTEND, 2);
		$classes					= array();
		
		if (preg_match($php_class, $phpcode, $regs) || preg_match($php_class_extend, $phpcode, $regs)) {

			list($name, $extends)=$this->getClassnamesFromRegs($regs);
			$fullname		= $regs[1];
			$end 				= 0;
			
			while (""!=$name) {
			
				$start 			= strpos($phpcode, $fullname);
				$remaining 	= substr($phpcode, $start+1);
				
				if (!($ok = preg_match($php_class, $remaining, $regs)))
					$ok = preg_match($php_class_extend, $remaining, $regs);
				
				if ($ok) {
					
					$end = strpos($phpcode, $regs[1]);
					$classes[$name] = substr($phpcode, $start, ($end-$start));
					list($classname, $extends)=$this->getClassnamesFromRegs($regs);
					$name			= $classname;
					$fullname = $regs[1];
					
				} else {
				
					$classes[$name] = substr($phpcode, $start);
					$name = "";
					
				}
				
				$phpcode = $remaining;
			}
			
		}		
		
		return $classes;
	} // end getClasscode
	
	/**
	* Extracts the classname and the name of the extended class from a result array 
	* provided by preg_match($this->PHP_CLASS, ...) or  preg_match($this->PHP_CLASS_EXTEND, ...)
	* @param	array	$regs
	* @return array $names	use list ( $classname, $extends ) = getClassnamesFromRegs($regs) 
	* @see		PHP_CLASS, PHP_CLASS_EXTEND, Setup()
	*/
	function getClassnamesFromRegs($regs) {
		if (!is_array($regs))
			return array("", "");
	
		if (isset($regs[3]))
			return array($regs[2], $regs[3]);
		else 
			return array($regs[2], "");
	} // end func getClassnamesFromRegs
	
} // end class phpdoc_class_analyser
?>