<?php
/**
* Looks for documented and undocumented functions within a block of php code
* @author	Ulf Wendel <ulf@redsys.de>
* @version 0.1alpha
*/
class PhpdocFunctionParser extends PhpdocVariableParser {

  /**
  * Array shared by several functions holding the information found in the code
  * @var    array $functions
  * @access private
  */
  var $functions = array();

	/**
	* Just calls Setup()
	* @see	Setup()
	*/
	function PhpdocFunctionParser() {
		$this->Setup();
	} // end constructor
	
	/**
	* Function analyser.
	* @param	string	$phpcode	code to analyse
	* @param	array		$pargraphs
	* @return	array		$class
	* @throws	runtime
	* @access	public
	*/
	function analyseFunctions($phpcode) {
	
		$this->functions = array();
		
		$paragraphs = $this->getPhpdocParagraphs($phpcode, "functions");
		reset($paragraphs["functions"]);
		while (list($k, $paragraph)=each($paragraphs["functions"])) 
			$this->analyseFunctionParagraph($paragraph);

		$this->addUndocumentedFunctions($phpcode);			
		
		return $this->functions;
	} // end func analyseFunctions

	/**
	* Analyses a singe function paragraph and adds the result to the function instance variable
	* @param	array	$paragraph
	* @see		functions
	*/	
	function analyseFunctionParagraph($paragraph) {
		
		$function = array();
		
		$name	= $paragraph["regs"][2];
		$function["name"] = $name;
		
		list($shortdesc, $fulldesc) = $this->getDescription($paragraph["phpdoc"]);
		$function["shortdesc"] 	= $shortdesc;
		$function["desc"] 			= $fulldesc;
		
		$tags = $this->getTags($paragraph["phpdoc"]);
		
		reset($tags);
		while (list($k, $tag)=each($tags)) {
			$tagname = $tag["tag"];
			
			switch ($tagname) {
			
				case "@param":
					$regexp = sprintf("/%s%s(%s)%s(%s)%s(.*)/",
															$this->C_VARTYPE,
															$this->C_SPACE_CR_OPTIONAL,
															$this->C_OBJECTNAME_OPTIONAL,
															$this->C_SPACE_CR,
															$this->PHP_VARNAME,
															$this->C_SPACE_CR_OPTIONAL
														);
					if (preg_match($regexp, $tag["value"], $regs)) {
						$function["param"][] = array (
																						"type"		=> ("object" == $regs[1]) ? "object ".$regs[2] : $regs[1],
																						"name"		=> $regs[3],
																						"desc"		=> $regs[4]
																				);
					} else 
						$function["param"]["invalid"][] = $tag;
					break;

				case "@return":
					$regexp = sprintf("/%s%s(%s)%s(%s)%s(.*)/",
															$this->C_VARTYPE,
															$this->C_SPACE_CR_OPTIONAL,
															$this->C_OBJECTNAME_OPTIONAL,
															$this->C_SPACE_CR,
															$this->PHP_VARNAME,
															$this->C_SPACE_CR_OPTIONAL
														);
					if (preg_match($regexp, $tag["value"], $regs)) {
						$function["return"] = array (
																						"type"		=> ("object" == $regs[1]) ? "object ".$regs[2] : $regs[1],
																						"name"		=> $regs[3],
																						"desc"		=> $regs[4]
																				);
					} else 
						$function["return"]["invalid"][] = $tag;
					break;

				case "@see":
					$regexp = sprintf("/%s/", $this->C_SEE_FUNCTION);
					if (preg_match_all($regexp, $tag["value"], $regs, PREG_SET_ORDER)) {
						reset($regs);
						while (list($k, $data)=each($regs)) 
							$function["see"]["functions"][] = substr($data[0], 0, -2);
						$tag["value"]=preg_replace($regexp, "", $tag["value"]);
					} 
					
					$regexp = sprintf("/%s/", $this->C_SEE_VAR);
					if (preg_match_all($regexp, $tag["value"], $regs, PREG_SET_ORDER)) {
						reset($regs);
						while (list($k, $data)=each($regs)) 
							$function["see"]["var"][] = $data[0];
					}
					break;
					
				case "@throws":
					$exceptions = explode(",", $tag["value"]);
					reset($exceptions);
					while (list($k, $exception)=each($exceptions)) 
						$function["throws"][] = trim($exception);
					break;
					
				case "@access":
					$regexp = sprintf("/%s/", $this->C_ACCESS);
					if (preg_match($regexp, $tag["value"], $regs)) 
						$function["access"] = $tag["value"];
					else 
						$function["ignored"][] = $tag;
					break;
					
				case "@version":
				case "@since":
					$function[substr($tagname, 1)] = $tag["value"];
					break;

				case "@abstract":					
					$function[substr($tagname, 1)] = true;
					break;
					
				default:
					$function["ignored"][] = $tag;
					break;
			}
		}
		
		$this->functions[$name] = $function;
		
	} // end func analyseFunctionParagraph
	
	/**
	* Looks for undocumented functions and add them to the parsing result
	* @param	string	$phpcode
	* @access	private
	*/
	function addUndocumentedFunctions($phpcode) {
		
		$regexp = "/". substr($this->PHP_FUNCTION, 2);
		preg_match_all($regexp, $phpcode, $paragraphs, PREG_SET_ORDER);
		
		reset($paragraphs);
		while (list($k, $paragraph)=each($paragraphs)) 
			if (!isset($this->functions[$paragraph[2]])) 
				$this->functions[$paragraph[2]]["undocumented"] = true;
				
	} // end func addUndocumentedFunctions
	
} // end class PhpdocFunctionParser
?>