<?php
/**
* Extract class variables and their documentation from phpcode
* @author   Ulf Wendel <ulf@redsys.de>
* @version  0.1
*/
class PhpdocVariableParser extends PhpdocParserObject {
	/**
	* Array holding all informations about the variables found in the php code
	* @var		array	$variables
	* @access	private
	*/
	var $variables = array();
	
	/**
	* Just calls Setup().
	* @see	Setup()
	*/
	function PhpdocVariableParser() {
		$this->Setup();
	} // end constructor
	
	/**
	* Looks for class variables
	* @param	string	$phpcode
	* @return	array		$variables
	* @access	public
	*/
  function analyseVariables($phpcode) {
		$this->variables = array();
		
		$paragraphs = $this->getPhpdocParagraphs($phpcode, "variables");
		reset($paragraphs["variables"]);
		while (list($k, $paragraph)=each($paragraphs["variables"]))
			$this->analyseVariableParagraph($paragraph);
			
		$this->addUndocumentedVariables($phpcode);		
		return $this->variables;
	} // end func analyseVariables
	
	/**
	* Analyses a single phpdoc paragraph describe one variable
	* @param	string	$phpdoc
	* @return	array
	* @access	private
	*/
	function analyseVariableParagraph($paragraph) {
		
		$variable = array();

		$name = substr($paragraph["regs"][2], 1);
		$variable["name"] = $name;
		
		list($shortdesc, $fulldesc) = $this->getDescription($paragraph["phpdoc"]);
		$variable["shortdesc"]	= $shortdesc;
		$variable["desc"]				= $fulldesc;
		
		$tags = $this->getTags($paragraph["phpdoc"]);
		
		reset($tags);
		while (list($k, $tag)=each($tags)) {
			$tagname = $tag["tag"];
			
			switch ($tagname) {
			
				case "@var":
					$regexp = sprintf("/%s%s(%s)%s%s%s(.*)/",
															$this->C_VARTYPE,
															$this->C_SPACE_CR_OPTIONAL,
															$this->C_OBJECTNAME_OPTIONAL,
															$this->C_SPACE_CR,
															$this->PHP_VARNAME,
															$this->C_SPACE_CR_OPTIONAL
														);
					if (preg_match($regexp, $tag["value"], $regs)) {
						$variable["var"] = array (
																						"type"		=> ("object" == $regs[1]) ? $regs[2] : $regs[1],
																						"desc"		=> $regs[3]
																				);
					} else 
						$variable["var"]["invalid"] = $tag;
					break;
					
				case "@access":
					$regexp = sprintf("/%s/", $this->C_ACCESS);
					if (preg_match($regexp, $tag["value"], $regs)) 
						$variable["access"] = $tag["value"];
					else 
						$variable["ignored"][] = $tag;
					break;
					
				case "@see":
					$regexp = sprintf("/%s/", $this->C_SEE_FUNCTION);
					if (preg_match_all($regexp, $tag["value"], $regs, PREG_SET_ORDER)) {
						reset($regs);
						while (list($k, $data)=each($regs)) 
							$variable["see"]["functions"][] = substr($data[0], 0, -2);
						$tag["value"]=preg_replace($regexp, "", $tag["value"]);
					} 
					
					$regexp = sprintf("/%s/", $this->C_SEE_VAR);
					if (preg_match_all($regexp, $tag["value"], $regs, PREG_SET_ORDER)) {
						reset($regs);
						while (list($k, $data)=each($regs)) 
							$variable["see"]["var"][] = $data[0];
					}
					break;
					
				case "@since":
				case "@version":
					$variables["since"] = $tag["value"];
					break;
					
				default:
					$variable["ignored"][] = $tag;
					break;
			}
		}
		
		$this->variables[$name] = $variable;
	} // end func analyseVariable
	
	/**
	* Adds undocumented Variables to the list of documented variables
	* @param	string	$phpcode
	* @access	private
	*/
	function addUndocumentedVariables($phpcode) {
		
		$regexp = "/".substr($this->PHP_VAR, 2);
		preg_match_all($regexp, $phpcode, $paragraphs, PREG_SET_ORDER);
		
		reset($paragraphs);
		while (list($k, $paragraph)=each($paragraphs)) {			
			$variable = substr($paragraph[2], 1);
			if (!isset($this->variables[$variable]))
				$this->variables[$variable]["undocumented"] = true;
		}
		
	} // end func addUndocumentedVariables
	
} // end class PhpdocVariableParser
?>