<?php
/**
* Base class of all Phpdoc Renderer.
*
* All Phpdoc Renderer are derived from this class. It contains some shared 
* methods and configuration values.
*
* @author		Ulf Wendel	<ulf@redsys.de>
* @version	0.1
*/
class PhpdocRendererObject extends PhpdocObject {
	
	/**
	* Array of strings with a certain number of spaces. 
	* 
	* The array is indexed by the number of spaces. It's accessed through
	* getIndent() and used to format text in <pre> paragraphs.
	*
	* @var		array	$indent
	* @access	private
	* @see		getIndent()
	*/
	var $indent = array();
	
	/**
	* Returns a string with of $lenght spaces. 
	* 
	* @param	int			$length
	* @return	string	$indent
	* @access	private
	*/
	function getIndent($length) {
		if (isset($this->indent[$length]))
			return $this->indent[$length];
		
		$indent = "";
		for ($i=0; $i<$length; $i++) 
			$indent.= " ";
			
		$this->indent[$length] = $indent;
		return $indent;
	} // end func getIndent

	/**
	* Renders the @see data as a list.
	* @param	mixed		$see
	* @return	string	$html
	*/
	function getSee($see) {
		if (!is_array($see)) 
			return "";
		
		$html = "";
		
		if (is_array($see["functions"])) {
			reset($see["functions"]);
			while (list($k, $functionname)=each($see["functions"])) 
				$html.= sprintf('<a href="#%s()">%s()</a>, ', $functionname, $functionname);
			$html = substr($html, 0, -2).$this->CR_HTML;
		} 
		if (is_array($see["variables"])) {
			reset($see["variables"]);
			while (list($k, $variable)=each($see["variables"])) 
				$html.= sprintf('<a href="#%s">%s</a>, ', $variable, $variable);
			$html = substr($html, 0, -2).$this->CR_HTML;
		}
		
		return $html;
	} // end func getSee
	
} // end class phpdoc_renderer
?>