<?php
/**
* Renders the variables in phpdoc
* @author 	Ulf Wendel <ulf@redsys.de>
* @version	0.1
*/ 
class PhpdocVariableRenderer extends PhpdocRendererObject {

	/**
	* Renders a summary of a variable
	* @param	string	$variable
	* @param	array		$data
	* @return	array		$summary
	* @access	public
	*/
	function getVariableSummary($variable, $data) {
		if (""==$variable || !is_array($data)) 
			return array("", "");
		
		if (isset($data["undocumented"])) {
			$type = sprintf("%-10s", "unknown");
		} else {
			$type = $data["var"]["type"];
		}
		$type 		= "<pre><code>$type</code></pre>";
		
		return array($type, $data["shortdesc"]);
	} // end func getVariableSummary
	
	/**
	* Renders a variable in detail
	* @param	string	$variable
	* @param	array		$data
	* @return	string	$html
	* @access	public
	* @throws	runtime_wrongparam
	*/
	function getVariableDetail($variable, $data) {
		if (""==$variable|| !is_array($data)) {
			$this->error = new exception_runtime_wrongparam("No variablename and/or no variabledata given.", 1, __FILE__, __LINE__, $this->AUTO_HANDLE_ERRORS);
			return "";
		}
		
		#$this->introspection("variable $variable", $data);
		
		$see		= $this->getSee($data["see"]);
		
		$html = sprintf('<a name="%s"></a>', $variable);
		$html.= sprintf('<dl><pre><dt><code>%s %s <b>%s</b>;</code>%s%s</dt></pre><dd>%s%s</dd>', 
											$data["access"],
											$data["var"]["type"],
											$variable,
											$this->CR_HTML,
											$this->checkBaseclass($data, $variable),
											$this->CR_HTML, 
											$data["undocumented"] ? "Undocumented." : $data["desc"]
										);
		
		if (""!=$data["since"]) {
			$html.= "<p><dl><dt>Since";
			$html.= $this->checkInheritedFieldFrom($data, "since", $variable);
			$html.=":</dt><dd>".$this->CR_HTML;
			$html.=	$data["since"]."</dd></dl>".$this->CR_HTML;
		}
		
		if (""!=$data["version"]) {
			$html.= "<p><dl><dt>Version";
			$html.= $this->checkInheritedFieldFrom($data, "version", $variable);
			$html.= ":</dt><dd>".$this->CR_HTML;
			$html.= $data["version"]."</dd></dl>".$this->CR_HTML;
		}
		
		if (""!=$see) {
			$html.= "<p><dl><dt>See also";
			$html.= $this->checkInheritedFieldFrom($data, "see", $variable);
			$html.= ":</dt><dd>".$this->CR_HTML;
			$html.= $see."</dd></dl>".$this->CR_HTML;
		}
		
		$html.= "</dl><hr>".$this->CR_HTML;
		return $html;
	} // end func getVariableDetail
	
	/**
	* Returns an HTML anchor if the field is inherited from another class
	* @param	array		$data
	* @param	string 	$field
	* @param	string	$varname
	* @return	string	$html
	* @access	private
	*/
	function checkInheritedFieldFrom($data, $field, $varname) {
		if (isset($data["overriden"][$field]["parentclass"])) {
			$html = sprintf(' [from: <a href="%s.html#%s">%s->%s</a>]',
														$data["overriden"][$field]["parentclass"],
														$varname,
														$data["overriden"][$field]["parentclass"],
														$varname
												);
		} else {
			$html = "";
		}
		
		return $html;
	} // end func checkInheritedFieldFrom

	/**
	* Returns an HTML anchor to the parent variable if available
	* @param	array		$data
	* @param	string	$varname
	* @return	string	$html
	* @access	private
	*/
	function checkBaseclass($data, $varname) {
		if (isset($data["baseclass"])) {
			$html = sprintf('[overrides: <a href="%s.html#%s">%s->%s</a>]',
												$data["baseclass"],
												$varname,
												$data["baseclass"],
												$varname,
												$this->CR_HTML
											);
		} else {
			$html = "";
		}
		
		return $html;
	} // end func checkBaseclass
	
} // end class PhpdocVariableRenderer
?>